@@ -61,7 +61,7 @@ def update_or_create(self,query=None,*args,files=None,unique_keys=None,max_queri
 
     #get updates
     updates=self._get_updates(*args,**kwargs)
-    logger.debug(f'update_or_create: Starting with query of length {len(query)}, {len(updates)} updates, and {"no" if files is None else (len(files) if type(files) in [list,tuple] else 0)} files')
+    logger.debug(f'update_or_create: Starting with query of length {len(query)}, {len(updates)} updates, and {"no" if files is None else (len(files) if type(files) in [list,tuple] else 1)} files')
 
     #Process dataframe tables (files)
     last_diff_t=0
@@ -75,14 +75,20 @@ def update_or_create(self,query=None,*args,files=None,unique_keys=None,max_queri
                 file=file[0]
             assert isinstance(file,pd.DataFrame), "Files should either be a list of dataframes or a list of [(DataFrame,{metadata}),...]"
 
+            total_rows=file.shape[0]
+            prev_seconds=0
             while file.shape[0]>0:
                 num_records_allowed=max_queries-len(query)
                 rows=self.df_to_records(file.iloc[:num_records_allowed],**cur_meta)
                 query.extend(rows)
 
+                file=file.iloc[num_records_allowed:]
                 if len(query)>=max_queries:
                     ids.extend(self.update_or_create(query=query,unique_keys=unique_keys,max_queries=max_queries,return_only_ids=True,**updates))
-                    file=file.iloc[num_records_allowed:]
+                    diff_seconds = (datetime.now() - start_t).total_seconds()
+                    seconds_per_row=(diff_seconds-prev_seconds)/len(query)
+                    logger.debug(f'update_or_create: Calling update_or_create on {len(rows)} rows out of {total_rows}: {((total_rows-file.shape[0])/total_rows):.1%} complete with {seconds_per_row*file.shape[0]:.1f} seconds remaining...')
+                    prev_seconds=diff_seconds
                     del rows
                     query.clear()
         del files
